#include <iostream>
#include "GLDGeoDisplay.h"
#include <string>
#include "TGLViewer.h"

#include "JSFSteer.h"
#include "TMath.h"

ClassImp(GLDGeoDisplay)

Int_t GLDGeoDisplay::fSerialNo=0;
TGeoManager *GLDGeoDisplay::fGeoManager=0;
GLDGeoDisplay *GLDGeoDisplay::fGLDGeoDisplay=0;

//________________________________________________________
GLDGeoDisplay::GLDGeoDisplay(EDispType dtype)
{
  fCanvas = new TCanvas("GLD", "Event Display", 200, 10, 800, 800);
  fCanvas ->SetFillColor(0);
  fView   = new TView(1);
  fZoomCanvas = kTRUE;
  fDispType   = dtype;
  fSerialNo   = 0;
  fGLDGeoDisplay = this;

  Initialize();

//  SetParameters();
//  BuildGeometry();
}

//________________________________________________________
GLDGeoDisplay::~GLDGeoDisplay()
{
  delete fCanvas;
}

//________________________________________________________
GLDGeoDisplay *GLDGeoDisplay::GetInstance()
{
  if( !fGLDGeoDisplay ) new GLDGeoDisplay;
  return fGLDGeoDisplay;
}


//________________________________________________________
void GLDGeoDisplay::Initialize()
{

  if (fGeoManager) delete fGeoManager;
  fGeoManager=new TGeoManager("gld","gld geometry");

//  if( fDispType == kXYView || fDispType == kXYViewMuon ) {
    fView->Top();
    fView->SetParallel();
     fCanvas->SetView(fView);
//   }

  
  TGeoMaterial  *mat=new TGeoMaterial("Al",26.98, 13, 2.7);
  Char_t tlevel=0;
  mat->SetTransparency(tlevel);
  fMed=new TGeoMedium("med",1,mat);

  fTop=fGeoManager->MakeBox("top",0,
            gJSF->Env()->GetValue("J4.EXPHall.HalfX",1000.0),
            gJSF->Env()->GetValue("J4.EXPHall.HalfY",1000.0), 
            gJSF->Env()->GetValue("J4.EXPHapp.HalfZ",1500.0) );
  fGeoManager->SetTopVolume(fTop);

  fView->SetRange(-300.0,-300.0,-300.0,300.0,300.0,300.0);
  fView->SetParallel();
  fView->Top();

  VTXGeometry();
  ITGeometry();
  TPCGeometry();
  CLXGeometry();
  if( HasMuon() ) {
    SOLGeometry();
    MUDGeometry();
  }

  fGeoManager->CloseGeometry();


#if 0
  TView *topview=new TView(1);
  topview->SetRange(-300.0,-300.0,-300.0,300.0,300.0,300.0);
  topview->TopView();
  topview->SetParallel();
  gPad->SetView(topview);
  gPad->Modified();
  gPad->Update();
#endif

#if 0
  top->Draw("ogl");
  TGLViewer *viewer=gPad->GetViewer3D("ogl");
  viewer->SetCurrentCamera(TGLViewer::kCameraOrthoXOY);
  //  gPad->Set  BackgroundColor(256);
  //  gVirtualGL->CaptureViewer(viewer, 1, "myfile.eps");
#endif
}
// =============================================
void GLDGeoDisplay::Draw()
{

//  fGeoManager->SetNsegments(96);

  std::cerr << "GLDGeoDisplay::Draw was called " << std::endl;
  
  if( !IsPerspective() ) {  
     fView->SetParallel(); 
     fCanvas->SetView(fView);
  }
  fTop->Draw("ogl");
  TGLViewer *viewer=(TGLViewer*)gPad->GetViewer3D("ogl");
  viewer->SetCurrentCamera(TGLViewer::kCameraOrthoXOY);

  if( !IsPerspective() ) {
     fView->SetParallel();
  }

}

// =============================================
void GLDGeoDisplay::Update()
{
  if( !IsPerspective() ) { fView->SetParallel(); fCanvas->SetView(fView); }
  fCanvas->Modified();
  fCanvas->Update();
  if( !IsPerspective() ) { fView->SetParallel(); }
}

// =============================================
void GLDGeoDisplay::Print()
{
//   if ( gROOT->IsBatch() ) {
     const Char_t *fdir=gJSF->Env()->GetValue("EVDISP.PrintPath","Figs/evdisp");
     Char_t fname[256];
     sprintf(fname,"%s_%4.4d.eps",fdir,gJSF->GetEventNumber());
     fCanvas->Print(fname);

}


//===============================================================
void GLDGeoDisplay::VTXGeometry()
{
  Int_t nlayers=gJSF->Env()->GetValue("J4VTX.NLayers",6);
  std::vector<double> layerR=gJSF->Env()->GetDValue("J4VTX.Layer.Radius", 
			"2.0 2.2 3.2 3.4 4.8 5.", nlayers);
  Double_t sensorLength=gJSF->Env()->GetValue("J4VTX.Sensor.Length",1.0);
  Double_t sensorThickness=gJSF->Env()->GetValue("J4VTX.Sensor.Thickness",0.005);
  Double_t epitaxThickness=gJSF->Env()->GetValue("J4VTX.Epitax.Thickness",0.0015);
  std::vector<int> layerNsensor=gJSF->Env()->GetIValue("J4VTX.NSensors",
			"13 13 20 20 20 20",nlayers);

  for(Int_t i=0;i<nlayers;i++) {
    double r=layerR[i];
    double halfZ=sensorLength*layerNsensor[i]*0.5;
    double rmax=r+sensorThickness+epitaxThickness;
    TGeoVolume *vol=fGeoManager->MakeTube("",0,r,rmax,halfZ);
    vol->SetLineColor(5);
    fSerialNo++;
    fTop->AddNode(vol,fSerialNo);
  }
}


//===============================================================
void GLDGeoDisplay::ITGeometry()
{
  Int_t nlayers=gJSF->Env()->GetValue("J4IT.NBarrel",4);
  std::vector<Double_t> layerR=gJSF->Env()->GetDValue("J4IT.Barrel.InnerRadius",
	"9.0  16.0   23.0  30.0", nlayers);
  Double_t thickness=gJSF->Env()->GetValue("J4IT.Layer.Thickness",0.056160);
  std::vector<Double_t> halfz=gJSF->Env()->GetDValue("J4IT.Barrel.HalfZ",
	"18.5  33.0  47.5  62.0",nlayers);
  
  for(Int_t i=0;i<nlayers;i++) {
    double r=layerR[i];
    double halfZ=halfz[i];
    double rmax=r+thickness;
    TGeoVolume *vol=fGeoManager->MakeTube("",0,r,rmax,halfZ);
    vol->SetLineColor(6);
    fSerialNo++;
    fTop->AddNode(vol,fSerialNo);
  }
}

//===============================================================
void GLDGeoDisplay::TPCGeometry()
{
  Double_t orInnerSupport=gJSF->Env()->GetValue("J4.SupportTube.OuterRadius",39.5);
  Double_t htInnerSupport=gJSF->Env()->GetValue("J4TPC.InnerSupportTube.HalfThickness",2.1075);
  Double_t irTPCDrift=orInnerSupport+2.0*htInnerSupport;
  Double_t orTPC=gJSF->Env()->GetValue("J4.TPC.OuterRadius", 206.00);
  Double_t htOuterSupport=gJSF->Env()->GetValue("J4TPC.OuterSupportTube.HalfThickness",4.117500);
  Double_t orTPCDrift=orTPC-2*htOuterSupport;

  Double_t hzTPC=gJSF->Env()->GetValue("J4.TPC.HalfZ",260.00);
  Double_t hzEndcap=gJSF->Env()->GetValue("J4TPC.Endcap.HalfThickness",1.00);
  Double_t hzPadPlane=gJSF->Env()->GetValue("J4TPC.PadPlane.HalfThickness",1.50);
  Double_t hzTPCDrift=hzTPC-2*(hzEndcap+hzPadPlane);

  TGeoVolume *vol=fGeoManager->MakeTube("",fMed,irTPCDrift,orTPCDrift,hzTPCDrift);
  //  TGeoTube *tube=new TGeoTube(irTPCDrift, orTPCDrift, hzTPCDrift);
  //  TGeoVolume *vol=new TGeoVolume("TPCDrift",tube,0);

  vol->SetLineColor(28);
  fSerialNo++;
  fTop->AddNode(vol,fSerialNo);

}

//===============================================================
void GLDGeoDisplay::CLXGeometry()
{
  Int_t ndiv=gJSF->Env()->GetValue("J4CLX.NTraps",12);
  Double_t halfz=gJSF->Env()->GetValue("J4CLX.HalfZ", 270.0);
  Double_t rmin=gJSF->Env()->GetValue("J4CLX.Barrel.InnerRadius",206.0);
  std::cerr << " rmin=" << rmin << std::endl;
  Double_t eml=gJSF->Env()->GetValue("J4CLX.EM.AbsLayerThickness",0.3)
           +gJSF->Env()->GetValue("J4CLX.EM.ActiveLayerThickness",0.2)
           +gJSF->Env()->GetValue("J4CLX.EM.FlexLayerThickness",0.1);
  Double_t hdl=gJSF->Env()->GetValue("J4CLX.HD.AbsLayerThickness",2.0)
           +gJSF->Env()->GetValue("J4CLX.HD.ActiveLayerThickness",0.4)
           +gJSF->Env()->GetValue("J4CLX.HD.FlexLayerThickness",0.1);
  Int_t   neml=gJSF->Env()->GetValue("J4CLX.EM.NLayers",30);
  Int_t   nhdl=gJSF->Env()->GetValue("J4CLX.HD.NLayers",48);
  Double_t thick=(eml*(Double_t)neml+hdl*(Double_t)nhdl);
  Double_t rmax=thick+rmin;
  Double_t emrmin=rmin;
  Double_t emrmax=rmin+eml*(Double_t)neml;
  Double_t hdrmin=emrmax;
  Double_t hdrmax=rmax;
  Double_t endir=gJSF->Env()->GetValue("J4LCX.Endcap.InnerRadius",39.9);
  Double_t enddz=gJSF->Env()->GetValue("J4CLX.BarrelEndcapGap",10.0);
  Double_t zcnt=enddz+halfz+thick*0.5;
  
  if( IsPerspective() ) {  // Show both Barrel and Endcap for 3D display

    DrawBlock(fTop, "CLXBarrelEM",ndiv, emrmin, emrmax, halfz, 0.0, 37);
    DrawBlock(fTop, "CLXBarrelHD",ndiv, hdrmin, hdrmax, halfz, 0.0, 39);
  
    DrawBlock(fTop, "CLXEndP",ndiv,endir,rmax, thick*0.5, zcnt, 37);
    DrawBlock(fTop, "CLXEndM",ndiv,endir,rmax, thick*0.5, -zcnt, 37);

  }
  else {
    DrawBlock(fTop, "CLXBarrelEM",ndiv, emrmin, emrmax, halfz, 0.0, 37);
    DrawBlock(fTop, "CLXBarrelHD",ndiv, hdrmin, hdrmax, halfz, 0.0, 39);

  }

  
}

// ==============================================================
void GLDGeoDisplay::MUDGeometry()
{
//
//  Barrel Muon
  Int_t    ndiv=gJSF->Env()->GetValue("J4MUD.NTraps",12);
  Double_t mudor=gJSF->Env()->GetValue("J4MUD.Height",765.0);
  Double_t mudir=gJSF->Env()->GetValue("J4MUD.Barrel.InnerRadius",450.0);
  Double_t mudfz=gJSF->Env()->GetValue("J4MUD.Endcap.FrontZ",485.0);
  Double_t mudthick=gJSF->Env()->GetValue("J4MUD.Endcap.Thickness",315);
  Double_t mudhz=mudfz+mudthick;

  DrawBlock(fTop, "MUDBarrel", ndiv, mudir, mudor, 0.5*mudhz, 0.5*mudhz, 4);
  DrawBlock(fTop, "MUDBarrel", ndiv, mudir, mudor, 0.5*mudhz, -0.5*mudhz, 5);
  

  Double_t endir=gJSF->Env()->GetValue("J4MUD.Endcap.InnerRadius",45.0);

  Double_t zcntpos=mudfz+0.5*mudthick;
  DrawBlock(fTop, "MUDEndP",ndiv, endir, mudir, mudthick*0.5, zcntpos, 4);
  DrawBlock(fTop, "MUDEndM",ndiv, endir, mudir, mudthick*0.5, -zcntpos, 4);
  
  Double_t endfir=gJSF->Env()->GetValue("J4MUD.FrontEndcap.OuterRadius",260.0);
  Double_t endfz=gJSF->Env()->GetValue("J4MUD.FrontEndcap.FrontZ",425.0);
  Double_t endfhw=(mudfz-endfz)*0.5;
  Double_t endfzc=endfz+endfhw;
  DrawBlock(fTop,"MUDFrontP",ndiv,endir,endfir, endfhw, endfzc, 4);
  DrawBlock(fTop,"MUDFrontP",ndiv,endir,endfir, endfhw, -endfzc, 4);
 

}
//
// ============================================================
void GLDGeoDisplay::DrawBlock(TGeoVolume *top, const char* name, Int_t ndiv, Double_t hmin, 
               Double_t hmax, Double_t halfz, Double_t zcnt, Int_t col)
{
//  Barrel Muon
  Double_t degtorad=TMath::Pi()/180.0;

  Double_t hangle=360.0/2.0/ndiv*degtorad;
//  Double_t maxr=hmax/TMath::Cos(hangle);
  Double_t x1=hmin*TMath::Tan(hangle);
  Double_t x2=hmax*TMath::Tan(hangle);
  Double_t halfh=(hmax-hmin)*0.5;
  
  TGeoVolume *vol=fGeoManager->MakeTube(name,fMed,hmin,hmax,halfz);
  TGeoRotation *rot=new TGeoRotation("mudrot",0.0, 90.0, 0.0);
  fSerialNo++;
  top->AddNode(vol,fSerialNo,rot);
  
  std::string trdname;
  trdname=std::string(name)+std::string("trd");
  TGeoVolume *trd=fGeoManager->MakeTrd1(trdname.data(),fMed,x1,x2,halfz,halfh);
  trd->SetLineColor(col);

  for( Int_t i=0;i<(Int_t)ndiv;i++) {
    Double_t phi=(360.0/ndiv)*(Double_t)i;
    TGeoRotation *rot=new TGeoRotation();
    rot->RotateY(-(phi-90.0));
    Double_t dx=(hmin+halfh)*TMath::Cos(phi*degtorad);
    Double_t dy=(hmin+halfh)*TMath::Sin(phi*degtorad);
    TGeoTranslation *tr=new TGeoTranslation(dx,zcnt,dy);
    TGeoCombiTrans *cmb=new TGeoCombiTrans(*tr, *rot);
    vol->AddNode(trd, i+1, cmb);    
  }
}

// ==============================================================
void GLDGeoDisplay::SOLGeometry()
{
  Double_t rmin=gJSF->Env()->GetValue("J4.SOL.InnerRadius",375.0);
  Double_t rmax=gJSF->Env()->GetValue("J4.SOL.OuterRadius",440.0);
  Double_t halfz=gJSF->Env()->GetValue("J4.SOL.HalfZ",475.0);
  
  TGeoVolume *sol=fGeoManager->MakeTube("sol",fMed,rmin,rmax,halfz);
//  TGeoRotation *rot=new TGeoRotation("rot",0.0, 90.0, 0.0);
  sol->SetLineColor(3);
  fSerialNo++;
  fTop->AddNode(sol,fSerialNo);
}
