#ifndef __GLDGeoDisplay__
#define __GLDGeoDisplay__

#include <string>
#include "TCanvas.h"
#include "TView.h"
#include "TGeoManager.h"
#include "TGeoVolume.h"
#include "TGeoMedium.h"


class GLDGeoDisplay
{
public:
  enum EDispType { k3DView, k3DViewMuon, kXYView, kXYViewMuon, kYZView , kYZViewMuon, kXZView, kXZViewMuon, kDemoView };
  GLDGeoDisplay(EDispType type=kXYView);
  virtual ~GLDGeoDisplay();

  void Draw();
  void Print();
  void Update();

  inline TCanvas *GetCanvas(){ return fCanvas;}

  void Initialize();

  void DrawBlock(TGeoVolume *top, const char* name, Int_t ndiv, Double_t hmin, 
               Double_t hmax, Double_t halfz, Double_t zcnt, Int_t col);

  void CLXGeometry();
  void VTXGeometry();
  void ITGeometry();
  void TPCGeometry();
  void SOLGeometry();
  void MUDGeometry();

//  void BuildGeometry();
//  void SetParameters();

  void SetZoomCanvas(Bool_t flag=kTRUE){ fZoomCanvas=flag; }

  void SetDispType(EDispType type) { fDispType=type;}
  inline EDispType GetDispType(){ return fDispType; }
  inline Bool_t HasMuon() {
    if ( fDispType == k3DViewMuon || fDispType == kXYViewMuon 
	 || fDispType == kYZViewMuon || fDispType == kXZViewMuon 
       || fDispType == kDemoView ) { return kTRUE; }
    else { return kFALSE; }
  }
  inline Bool_t IsPerspective() {
    if ( fDispType == k3DViewMuon || fDispType == k3DViewMuon || kDemoView ) { return kTRUE; }
    else { return kFALSE; }
  }


  static GLDGeoDisplay *GetInstance();
protected:
  static GLDGeoDisplay *fGLDGeoDisplay;
  TCanvas *fCanvas;
  TView   *fView;
  static Int_t   fSerialNo;
  static TGeoManager *fGeoManager;
  TGeoVolume    *fTop;
  TGeoMedium    *fMed;

  Bool_t fZoomCanvas;
  EDispType fDispType;

  ClassDef(GLDGeoDisplay,0) // GLDGeoDisplay

};

#endif
