#include <iostream>
#include "GLDGeometry.h"
#include "TTUBE.h"
#include "TBRIK.h"
#include "TRotMatrix.h"
// include <strstream>
#include <sstream>
#include "TFile.h"
#include "TSystem.h"
#include "TMath.h"
#include "JSFSteer.h"
#include "TROOT.h"

ClassImp(GLDGeometry)

GLDGeometry *GLDGeometry::fGLDGeometry=0;

//________________________________________________________
GLDGeometry::GLDGeometry(EDispType dtype)
{
  fCanvas = new TCanvas("GLD", "Event Display", 400, 10, 800, 800);
  fCanvas ->SetFillColor(0);
  fView   = new TView(1);
  fDispType   = dtype;
  fGLDGeometry=this;
  SetParameters();
  fNeedClear=kFALSE;
  fMessage = 0;

  /*
  fColorCode[std::string("TPC")]=183;
  fColorCode[std::string("CLX")]=170;
  fColorCode[std::string("MUON")]=179;
  */
  fColorCode[std::string("TPC")]=166;
  fColorCode[std::string("CLXEM")]=194;
  fColorCode[std::string("CLXHD")]=174;
  fColorCode[std::string("MUD")]=203;

  fColorCode[std::string("electron")]=2; // Red
  fColorCode[std::string("muon")]=106; // purple
  fColorCode[std::string("photon")]=6; // 
  fColorCode[std::string("proton")]=8; // green
  fColorCode[std::string("neutron")]=7; // cyan
  fColorCode[std::string("neutrino")]=33; // grey;
  fColorCode[std::string("pion")]=9; //  dark blue
  fColorCode[std::string("kaon")]=46; // brown
  fColorCode[std::string("k0L")]=46; // brown
  fColorCode[std::string("undefined")]=12;

  BuildGeometry();

}

//________________________________________________________
GLDGeometry::~GLDGeometry()
{
  delete fCanvas;
}

//________________________________________________________
void GLDGeometry::BuildGeometry()
{

  //   fView->SetRange(-360.,-360.,-360.,+360.,+360.,+360.);
  Double_t maxsize=250.0;
  fView->SetRange(-maxsize, -maxsize, -maxsize, maxsize, maxsize, maxsize);
  //  Int_t ierr ;
  fView->Top();
  //  fView->UnZoom();
  fView->SetParallel();
  fCanvas->SetView(fView);

  TBRIK *exphall = new TBRIK("EXPHALL", "EXPHALL", "void", maxsize, maxsize, maxsize);
   fTop = new TNode("TOP","TOP",exphall);
   fTop->SetVisibility(0);
   fTop->cd();

   BuildTPC();
   BuildCLX();

   fTop->cd();
}

//________________________________________________________
GLDGeometry *GLDGeometry::GetInstance()
{
  if( !fGLDGeometry ) new GLDGeometry;
  return fGLDGeometry;
}


//===============================================================
void GLDGeometry::BuildVTX()
{
#if 0
  Int_t nlayers=gJSF->Env()->GetValue("J4VTX.NLayers",6);
  std::vector<double> layerR=gJSF->Env()->GetDValue("J4VTX.Layer.Radius", 
			"2.0 2.2 3.2 3.4 4.8 5.", nlayers);
  Double_t sensorLength=gJSF->Env()->GetValue("J4VTX.Sensor.Length",1.0);
  Double_t sensorThickness=gJSF->Env()->GetValue("J4VTX.Sensor.Thickness",0.005);
  Double_t epitaxThickness=gJSF->Env()->GetValue("J4VTX.Epitax.Thickness",0.0015);
  std::vector<int> layerNsensor=gJSF->Env()->GetIValue("J4VTX.NSensors",
			"13 13 20 20 20 20",nlayers);

  for(Int_t i=0;i<nlayers;i++) {
    double r=layerR[i];
    double halfZ=sensorLength*layerNsensor[i]*0.5;
    double rmax=r+sensorThickness+epitaxThickness;
//    vol->SetLineColor(5);
//    fSerialNo++;
//    fTop->AddNode(vol,fSerialNo);
  }
#endif
}


#if 0
//===============================================================
void GLDGeoDisplay::ITGeometry()
{
  Int_t nlayers=gJSF->Env()->GetValue("J4IT.NBarrel",4);
  std::vector<Double_t> layerR=gJSF->Env()->GetDValue("J4IT.Barrel.InnerRadius",
	"9.0  16.0   23.0  30.0", nlayers);
  Double_t thickness=gJSF->Env()->GetValue("J4IT.Layer.Thickness",0.056160);
  std::vector<Double_t> halfz=gJSF->Env()->GetDValue("J4IT.Barrel.HalfZ",
	"18.5  33.0  47.5  62.0",nlayers);
  
  for(Int_t i=0;i<nlayers;i++) {
    double r=layerR[i];
    double halfZ=halfz[i];
    double rmax=r+thickness;
    TGeoVolume *vol=fGeoManager->MakeTube("",0,r,rmax,halfZ);
    vol->SetLineColor(6);
    fSerialNo++;
    fTop->AddNode(vol,fSerialNo);
  }
}

#endif

//________________________________________________________
void GLDGeometry::BuildTPC()
{

  Double_t orInnerSupport=gJSF->Env()->GetValue("J4.SupportTube.OuterRadius",39.5);
  Double_t htInnerSupport=gJSF->Env()->GetValue("J4TPC.InnerSupportTube.HalfThickness",2.1075);
  Double_t irTPCDrift=orInnerSupport+2.0*htInnerSupport;
  Double_t orTPC=gJSF->Env()->GetValue("J4.TPC.OuterRadius", 206.00);
  Double_t htOuterSupport=gJSF->Env()->GetValue("J4TPC.OuterSupportTube.HalfThickness",4.117500);
  Double_t orTPCDrift=orTPC-2*htOuterSupport;

  Double_t hzTPC=gJSF->Env()->GetValue("J4.TPC.HalfZ",260.00);
  Double_t hzEndcap=gJSF->Env()->GetValue("J4TPC.Endcap.HalfThickness",1.00);
  Double_t hzPadPlane=gJSF->Env()->GetValue("J4TPC.PadPlane.HalfThickness",1.50);
  Double_t hzTPCDrift=hzTPC-2*(hzEndcap+hzPadPlane);


   TTUBE *tpcis=new TTUBE("TPCIS","TPCIS","void",irTPCDrift,irTPCDrift,hzTPCDrift);
   tpcis->SetLineColor(GetColorCode("TPC"));
   TTUBE *tpcos=new TTUBE("TPCOS","TPCOS","void",orTPCDrift,orTPCDrift,hzTPCDrift);
   tpcos->SetLineColor(GetColorCode("TPC"));

   if( !IsPerspective() ) {
     tpcis->SetNumberOfDivisions(512);
     tpcos->SetNumberOfDivisions(512);
    }

   fTop->cd();
   new TNode("NTPCIS","NTPCIS",tpcis);
   new TNode("NTPCOS","NTPCOS",tpcos);


}

//________________________________________________________
void GLDGeometry::BuildCLX()
{
  //  Int_t ndiv=gJSF->Env()->GetValue("J4CLX.NTraps",12);
  Double_t halfz=gJSF->Env()->GetValue("J4CLX.HalfZ", 270.0);
  Double_t rmin=gJSF->Env()->GetValue("J4CLX.Barrel.InnerRadius",206.0);
  Double_t eml=gJSF->Env()->GetValue("J4CLX.EM.AbsLayerThickness",0.3)
           +gJSF->Env()->GetValue("J4CLX.EM.ActiveLayerThickness",0.2)
           +gJSF->Env()->GetValue("J4CLX.EM.FlexLayerThickness",0.1);
  Double_t hdl=gJSF->Env()->GetValue("J4CLX.HD.AbsLayerThickness",2.0)
           +gJSF->Env()->GetValue("J4CLX.HD.ActiveLayerThickness",0.4)
           +gJSF->Env()->GetValue("J4CLX.HD.FlexLayerThickness",0.1);
  Int_t   neml=gJSF->Env()->GetValue("J4CLX.EM.NLayers",30);
  Int_t   nhdl=gJSF->Env()->GetValue("J4CLX.HD.NLayers",48);
  Double_t emthick=eml*(Double_t)neml;
  Double_t hdthick=hdl*(Double_t)nhdl;
  Double_t thick=emthick+hdthick;
  Double_t rotang=360.0/(gJSF->Env()->GetValue("J4CLX.NTraps",12)*2);
  Double_t emrmin=rmin/TMath::Cos(TMath::Pi()/180.0*rotang);
  Double_t rmax=thick+rmin;
  Double_t emrmax=rmin+eml*(Double_t)neml;
  Double_t hdrmin=emrmax;
  Double_t hdrmax=rmax;
  Double_t endir=gJSF->Env()->GetValue("J4LCX.Endcap.InnerRadius",39.9);
  Double_t enddz=gJSF->Env()->GetValue("J4CLX.BarrelEndcapGap",10.0);
  Double_t emzcnt=halfz+enddz+emthick*0.5;
  Double_t hdzcnt=halfz+enddz+emthick+0.5*hdthick;
  //  Double_t zcnt=enddz+halfz+thick*0.5;
  TRotMatrix *clxorotp  = new TRotMatrix("clxorot","clxorot", 90.0,rotang,90.,90.0+rotang,0,0 );

  if( !IsTopView() )  // Show both Barrel and Endcap for 3D display
    {

    TTUBE *clxbhd = new TTUBE("CLXBHD","CLXBHD","void",hdrmin, hdrmax, halfz);
    clxbhd->SetNumberOfDivisions(12);
    clxbhd->SetLineColor(GetColorCode("CLXHD"));
    TTUBE *clxbem = new TTUBE("CLXBEM","CLXBEM","void",emrmin, emrmax, halfz);
    clxbem->SetNumberOfDivisions(12);
    clxbem->SetLineColor(GetColorCode("CLXEM"));


    TTUBE *clxepem = new TTUBE("CLXEPEM","CLXEPEM","void",endir, rmax, emthick*0.5);
    clxepem->SetNumberOfDivisions(12);
    clxepem->SetLineColor(GetColorCode("CLXEM"));
    TTUBE *clxephd = new TTUBE("CLXEPHD","CLXEPHD","void",endir, rmax, hdthick*0.5);
    clxephd->SetNumberOfDivisions(12);
    clxephd->SetLineColor(GetColorCode("CLXHD"));

    fTop->cd();

    new TNode("NCLXBHD","NCLXBHD",clxbhd,0.,0.,0., clxorotp,"");
    new TNode("NCLXBEM","NCLXBEM",clxbem,0.,0.,0., clxorotp,"");
    new TNode("NCLXEPEM","NCLXEPEM",clxepem,0.,0., emzcnt,clxorotp,"");
    new TNode("NCLXEMEM","NCLXEMEM",clxepem,0.,0.,-emzcnt,clxorotp,"");
    new TNode("NCLXEPHD","NCLXEPHD",clxephd,0.,0., hdzcnt,clxorotp,"");
    new TNode("NCLXEMHD","NCLXEMHD",clxephd,0.,0.,-hdzcnt,clxorotp,"");

 

  }
  else {
    TTUBE *clxbhd = new TTUBE("CLXBHD","CLXBHD","void",hdrmin, hdrmax, halfz);
    clxbhd->SetNumberOfDivisions(12);
    clxbhd->SetLineColor(GetColorCode("CLXHD"));
    TTUBE *clxbem = new TTUBE("CLXBEM","CLXBEM","void",emrmin, emrmax, halfz);
    clxbem->SetNumberOfDivisions(12);
    clxbem->SetLineColor(GetColorCode("CLXEM"));


    fTop->cd();


    new TNode("NCLXBHD","NCLXBHD",clxbhd,0.,0.,0.,clxorotp,"");
    new TNode("NCLXBEM","NCLXBEM",clxbem,0.,0.,0.,clxorotp,"");
   

  }

}


//________________________________________________________
void GLDGeometry::SetParameters()
{

}

//________________________________________________________
void GLDGeometry::Draw()
{
  if( fView ) { delete fView; }
  if( fMessage ) { delete fMessage; }

  fCanvas->cd();
  fCanvas->Clear();

  fView=new TView(1);
  Int_t ierr=0;
  if( IsPerspective() ) {
    Double_t maxsize=300.0;
    fView->SetRange(-maxsize, -maxsize, -maxsize, maxsize, maxsize, maxsize);
    fView->SetView(0.0, 80.0, 80.0, ierr);
  }
  else {
    Double_t maxsize=275.0;
    fView->SetRange(-maxsize, -maxsize, -maxsize, maxsize, maxsize, maxsize);
    fView->SetParallel();
    //    std::cerr << " fDispType=" << fDispType << std::endl;
    if( fDispType == kXYView || fDispType == kXYViewMuon ) {
      fView->Top();
      fView->SetParallel();
    }
    else {
      fView->Side();
      fView->SetView(0.0, 90.0, -90.0, ierr);
      fView->SetParallel();
    }
  }
  fCanvas->SetView(fView);

  fTop->Draw("pad same");
   
  TFile *ifile=gJSF->GetInput();
  std::string infname(ifile->GetName());
  //  std::ostrstream sevno;
  std::stringstream sevno;
  sevno << " Event " << gJSF->GetEventNumber() 
	<< "  File : " << infname << std::ends;
  
  fMessage=new TLatex(-0.98, -0.98, sevno.str().data());
  fMessage->SetTextFont(11);
  fMessage->SetTextSize(0.03);
  fMessage->Draw("same");

 

}

//________________________________________________________
void GLDGeometry::Print()
{
   if ( gROOT->IsBatch() ) { 
     const Char_t *fdir=gJSF->Env()->GetValue("EVDISP.PrintPath","Figs/evdisp");
     Char_t fname[256];
     sprintf(fname,"%s_%4.4d.eps",fdir,gJSF->GetEventNumber());
     fCanvas->Print(fname);
     Char_t *opt="-resample 100x100 -quality 100 -resize 800x800 -sharpen 5 ";
     sprintf(fname,"convert %s_%4.4d.eps %s %s_%4.4d.png",fdir,gJSF->GetEventNumber(),
	     opt, fdir,gJSF->GetEventNumber());
     gSystem->Exec(fname);

   }
}

