#ifndef __GLDGeometry__
#define __GLDGeometry__


#include "TCanvas.h"
#include "TNode.h"
#include "TView.h"
#include "TLatex.h"
#include <map>
#include <string>

class GLDGeometry
{
public:
  enum EDispType { k3DView, k3DViewMuon, kXYView, kXYViewMuon, 
		   kYZView , kYZViewMuon, kXZView, kXZViewMuon };
  GLDGeometry(EDispType type=kXYView);
  virtual ~GLDGeometry();

  void Print();
  void Draw();
  inline TCanvas *GetCanvas(){ return fCanvas;}
  inline TNode   *GetTopNode(){ return fTop;}

  void BuildGeometry();
  void SetParameters();

  void BuildVTX();
  void BuildIT();
  void BuildCLX();
  void BuildTPC();

  void SetDispType(EDispType type) { fDispType=type;}
  inline EDispType GetDispType(){ return fDispType; }
  inline Bool_t HasMuon() {
    if ( fDispType == k3DViewMuon || fDispType == kXYViewMuon
         || fDispType == kYZViewMuon || fDispType == kXZViewMuon ) { return kTRUE; }
    else { return kFALSE; }
  }
  inline Bool_t IsPerspective() {
    if ( fDispType == k3DView || fDispType == k3DViewMuon ) { return kTRUE;}
    else { return kFALSE; }
  }
  inline Bool_t IsTopView() {
    if ( fDispType == kXYView || fDispType == kXYViewMuon ) { return kTRUE;}
    else { return kFALSE; }
  }

  inline Int_t GetColorCode(const Char_t *name){ return fColorCode[std::string(name)]; }

  static GLDGeometry *GetInstance();


protected:
  static GLDGeometry *fGLDGeometry;
  TCanvas *fCanvas;
  TNode   *fTop;
  TView   *fView;
  TLatex  *fMessage;
  EDispType fDispType;
  Bool_t  fNeedClear;
  std::map<std::string, int> fColorCode;

  ClassDef(GLDGeometry,0) // GLD Geometry

};

#endif

