
//****************************************************
//*
//*  Dummy UserAnalysis Script to be used with gui.C 
//*  Functions in this scripts are called by GUIMainMacro.C  
//*  
//*  In this script, global functions, UserInitialize() nad UserAnalysis()
//*  Must be defined.  UserInitialize() is called at job initialization 
//*  to initialize histograms, etc.  It is also called when reset historgam
//*  menu is selected from JSF Control Panel.  UserAnalysis() is called 
//*  during the event loop after executing Process functions of JSF modules
//*  and display event daia but before display histogram.  See GetEvent()
//*  function in GUIMainMacro.C, where UserAnalysis() is called.
//*
//*  UserSetOptions() and UserTerminate() may be defined in this file.
//*  UserSetOptions() are called before declaration of JSF modules.
//*  It can be used to set parameters optional for user analsis.
//*  UserTerminate() is called during the JSF termination process.
//*
//*  When runmode=4, UserModuleDefine() must be defined in this file.
//*  It is used to define JSF modules specific to user analysis.  
//*  
//*$Id: UserAnalysis.C,v 1.2 2006/05/11 13:46:55 ono Exp $
//*
//****************************************************

//_________________________________________________________
void UserInitialize()
{
//----------------------------------
// set parameters
//----------------------------------
#if 0
   //   S4TPCPadHit::SetSpatialResolution("experimental"); // default is "flat".

   // Set IP constraint ....
   // S4TPCTrack::SetIPConstraint(kTRUE);

   // Artificial bunch tagging
   Double_t t0 = 0.;
   S4TPCTrack::SetT0pedestal(t0); // [ns]
#endif

   Int_t vtype=gJSF->Env()->GetValue("EVDISP.View",1);
   if( vtype == 2 ) {
     new GLDGeometry(GLDGeoDisplay::kXYView);
   }
   else if ( vtype == 3 ) {
     new GLDGeometry(GLDGeoDisplay::kYZView);
   }
   else {
     new GLDGeometry(GLDGeoDisplay::k3DView);
   }

   S4CALHitBuf::SetHitMakingScheme(1); // idealistic (smeared 1 hit/1 exact hit)

//----------------------------------
// set subtrackers to hybtrackmaker 
//----------------------------------

  S4VTXTrackMaker *vtxtrackmaker = (S4VTXTrackMaker *)gJSF->FindModule("S4VTXTrackMaker");
  S4ITTrackMaker  *ittrackmaker  = (S4ITTrackMaker *)gJSF->FindModule("S4ITTrackMaker");
  S4TPCTrackMaker *tpctrackmaker = (S4TPCTrackMaker *)gJSF->FindModule("S4TPCTrackMaker");
  S4HYBTrackMaker *htrackmaker   = (S4HYBTrackMaker *)gJSF->FindModule("S4HYBTrackMaker");
  htrackmaker->SetMainTrackMakerPtr(tpctrackmaker);
  htrackmaker->AddSubTrackMakerPtr(ittrackmaker);
  htrackmaker->AddSubTrackMakerPtr(vtxtrackmaker);
}

//_________________________________________________________
void UserAnalysis()
{
   S4PFOMakerBuf *pfombp = (S4PFOMakerBuf *)
                              gJSF->FindModule("S4PFOMaker")->EventBuf();
//   pfombp->DebugPrint();  // Remove comment to print information of PFOs.
   
   GLDGeometry *fGeometry=GLDGeometry::GetInstance();
   
   fGeometry->Draw();

#if 1
   DrawPFOs(); // draw option ("p","t") = (w/ prehits, w/ tracks)
#else
   DrawPFOs("p"); // draw option ("p","t") = (w/ prehits, w/ tracks)
#endif
 
   fGeometry->Print();

}

static Int_t        color  = 0;

//_________________________________________________________
void DrawPFOs(const Char_t *opt = "")
{
   TDirectory *last = gDirectory;
//----------------------------------
// initialize event display
//----------------------------------
  

   //x   omudnodep->Draw("pad same");

   //
   GLDGeometry *fGeometry=GLDGeometry::GetInstance();
   TObjArray *pfosp = ((S4PFOMakerBuf *)gJSF->FindModule("S4PFOMaker")
                         ->EventBuf())->GetOutputBufPtr();


   if (pfosp && pfosp->GetEntries()) {
      TIter next(pfosp);
      S4PFObject *pfp = 0;
      while ((pfp = (S4PFObject *)next())) {
	Int_t pdgc = pfp->GetPDGCode();
	switch (TMath::Abs(pdgc)) {
	case 11:  color=fGeometry->GetColorCode("electron"); break ;;
	case 13:  color=fGeometry->GetColorCode("muon"); break ;;
	case 22:  color=fGeometry->GetColorCode("photon"); break ;;
	case 211: color=fGeometry->GetColorCode("pion"); break ;;
	case 321: color=fGeometry->GetColorCode("kaon"); break ;;
	case 130: color=fGeometry->GetColorCode("k0L"); break ;;
	case 2212:color=fGeometry->GetColorCode("proton"); break ;;
	case 2112:color=fGeometry->GetColorCode("neutron"); break ;;
	case 12:  color=fGeometry->GetColorCode("neutrino"); break ;;
	case 14:  color=fGeometry->GetColorCode("neutrino"); break ;;
	case 16:  color=fGeometry->GetColorCode("neutrino"); break ;;
	default:  color=fGeometry->GetColorCode("undefined"); break ;;
	}
	pfp->Draw(color,opt);
        // DrawPFObject(pfp, color);
      }
   }

   if( GLDGeometry::GetInstance()->HasMuon() ) {

   S4MUDMCTruth    *ehit    = (S4MUDMCTruth*)gJSF->FindModule("S4MUDMCTruth");
   S4MUDMCTruthBuf *mctbuf  = (S4MUDMCTruthBuf*)ehit->EventBuf();
   TObjArray       *mudhits = mctbuf->GetExactHitsPtr();
 
   Bool_t showmud=kFALSE;  
   if ( mudhits && showmud ) {
     TIter mudnext( mudhits );
     S4MUDExactHit *mudhit = 0;
     while( (mudhit = (S4MUDExactHit*)mudnext()) ) {

       TPolyMarker3D *pm3dpp = new TPolyMarker3D(1);
       
       pm3dpp->SetBit(kCanDelete);
       pm3dpp->SetMarkerStyle(20);
       //       pm3dpp->SetMarkerColor(4);
       pm3dpp->SetMarkerSize(0.5);

       Int_t pdgc = TMath::Abs(mudhit->GetPDGCode());

       //if ( pdgc != 12 && pdgc !=14 && pdgc != 16 )
       //  std::cout << "MUD ExactHit PDG code = " << pdgc << std::endl;

	Int_t pdgc = pfp->GetPDGCode();
	switch (TMath::Abs(pdgc)) {
	case 11:  color=fGeometry->GetColorCode("electron"); break ;;
	case 13:  color=fGeometry->GetColorCode("muon"); break ;;
	case 22:  color=fGeometry->GetColorCode("photon"); break ;;
	case 211: color=fGeometry->GetColorCode("pion"); break ;;
	case 321: color=fGeometry->GetColorCode("kaon"); break ;;
	case 130: color=fGeometry->GetColorCode("k0L"); break ;;
	case 2212:color=fGeometry->GetColorCode("proton"); break ;;
	case 2112:color=fGeometry->GetColorCode("neutron"); break ;;
	case 12:  
	case 14:
	case 16:
	  pm3dpp->SetMarkerSize(0.2);	  
	  color=fGeometry->GetColorCode("neutrino"); break ;;
	default:  color=fGeometry->GetColorCode("undefined"); break ;;
	}

       pm3dpp->SetMarkerColor(color); 
       pm3dpp->SetPoint(0, mudhit->GetPosition().X(),
                           mudhit->GetPosition().Y(),
                           mudhit->GetPosition().Z());

       pm3dpp->Draw("same");       
     }
   }
   }

   gPad->Modified();
   gPad->Update();
   //   GLDGeoDisplay::GetInstance()->Update();
  

   //   std::cerr << " End of DrawPFO .. " << std::endl;
   
#if 0
   if ( gROOT->IsBatch() ) { 
     Char_t fname[256];
     sprintf(fname,"evdisp_%d.eps",gJSF->GetEventNumber());
     cvp->Print(fname);
   }
#endif

   last->cd();
}

//_________________________________________________________
void DrawHist()
{
}

//_________________________________________________________
/*
void DrawPFObject(S4PFObject *pfp)
{
  S4HYBTrack *trkPtr=pfp->GetHYBTrackPtr();
  S4CALCluster *clsPtr=pfp->GetClusterPtr();


}
*/
