Int_t showGLD()
{
	TGeoManager *geoMgr = new TGeoManager("geoMgr","ILC-GLD Geometry Manager");

	TGeoMaterial *matVac = new TGeoMaterial("Vacuum",0,0,0);
	TGeoMedium *medVac = new TGeoMedium("Vacuum",1,matVac);

	const Bool_t viewVTX = 1;
	const Bool_t viewIT  = 1;
	const Bool_t viewTPC = 1;
	const Bool_t viewCLX = 1;
	const Bool_t viewSOL = 1;
	const Bool_t viewMUD = 0;
	Char_t transparency=0;

	//	const Double_t expHallSize = 1000.0;
	const Double_t expHallSize = 600.0;
	if( viewMUD ) { expHallSize = 1000.0; }
	TGeoVolume *volExpHall = geoMgr->MakeBox("ExpHall",medVac,expHallSize,expHallSize,expHallSize);
	volExpHall->SetLineColor(kMagenta);
	geoMgr->SetTopVolume(volExpHall);
	// geoMgr->SetTopVisible();


        TGeoElementTable *etable=geoMgr->GetElementTable();
        TGeoMaterial *pmSilicon=new TGeoMaterial("Silicon",etable->FindElement("Argon"),2.33);
        pmSilicon->SetTransparency(transparency);
	TGeoMedium   *pmVTX=new TGeoMedium("VTX",1,pmSilicon);
	TGeoMedium   *pmIT =new TGeoMedium("IT",2,pmSilicon);


        TGeoMaterial *pmArgon=new TGeoMaterial("Argon",etable->FindElement("Argon"),1.78e-3);
        pmArgon->SetTransparency(transparency);

        TGeoMaterial *pmCarbon=new TGeoMaterial("Carbon",etable->FindElement("Carbon"),2.265);
        pmCarbon->SetTransparency(transparency);
	TGeoMedium   *pmTPCGas=new TGeoMedium("TPCGas",3,pmArgon);
	TGeoMedium   *pmTPCSupport=new TGeoMedium("TPCSupport",4,pmArgon);

	TGeoMaterial *pmTungsten=new TGeoMaterial("Tungsten", etable->FindElement("Tungsten"), 19.3);
	pmTungsten->SetTransparency(transparency);
        TGeoMedium   *pmEMCAL=new TGeoMedium("EMCAL",5,pmTungsten);

	TGeoMaterial *pmIron=new TGeoMaterial("Iron",etable->FindElement("Iron"),7.87);
	pmIron->SetTransparency(transparency);
        TGeoMedium   *pmHDCAL=new TGeoMedium("HDCAL",6,pmIron);
        TGeoMedium   *pmMUD=new TGeoMedium("MUD",7,pmIron);
	TGeoMedium   *pmCoil=new TGeoMedium("Coil",8,pmIron);



	if(viewVTX) {

		// VTX detectors
		const Int_t nVtxLayers = 6;
		Double_t rVtxLayer[nVtxLayers] = {2.0, 2.2, 3.2, 3.4, 4.8, 5.0};
		Int_t nVtxSensors[nVtxLayers] = {13, 13, 20, 20, 20, 20};
		const Double_t vtxSensorLength = 1.0;
		const Double_t vtxSensorThickness = 0.005;
		const Double_t dxyzMargin = 0.001;
		Double_t dzVtxLayer[nVtxLayers] = {0};
		Double_t vtxHalfZ[nVtxLayers] = {6.5, 6.5, 10.0, 10.0, 10.0, 10.0};
		for (Int_t i=0; i<nVtxLayers; i++) dzVtxLayer[i] = nVtxSensors[i]*vtxSensorLength + 4*dxyzMargin;

		for (Int_t i=0; i<nVtxLayers; i++) { // barrel VTX layers
			TGeoTube *vtxTube = new TGeoTube(Form("vtxTube%i",i),rVtxLayer[i],rVtxLayer[i] + vtxSensorThickness, vtxHalfZ[i]);
			TGeoVolume *vtxLayer = new TGeoVolume(Form("vtxLayer%i",i),vtxTube, pmVTX);
			vtxLayer->SetLineColor(3);
			volExpHall->AddNode(vtxLayer, i);
		}

		// endcap VTX
		TGeoTube *vtxEndcapTube = new TGeoTube("vtxEndcapTube", 4.0, 5.8, vtxSensorThickness);
		TGeoVolume *vtxEndcap = new TGeoVolume("vtxEndcap", vtxEndcapTube, pmVTX);
		vtxEndcap->SetLineColor(3);
		Double_t vtxEndcapZpos[2] = {12.0, 12.2};
		for (Int_t i=0; i<2; i++) {
			volExpHall->AddNode(vtxEndcap, 2*i+1, new TGeoTranslation(0.0, 0.0, vtxEndcapZpos[i]));
			volExpHall->AddNode(vtxEndcap, 2*i+2, new TGeoTranslation(0.0, 0.0, -vtxEndcapZpos[i]));
		}

	}

	if (viewIT) {

		// IT detectors
		const Int_t itNBarrel = 4;
		const Int_t itNEndcap = 7;
		const Double_t itLayerThickness = 0.056160;

		Double_t itBarrelHalfZ[itNBarrel] = {18.5, 33.0, 47.5, 62.0};
		Double_t itBarrelInnerR[itNBarrel] = {9.0, 16.0, 23.0, 30.0};
		Double_t itEndcapRmax[itNEndcap] = {7.6, 14.0, 21.0, 28.0, 38.0, 38.0, 38.0};
		Double_t itEndcapRmin[itNEndcap] = {2.4, 3.2, 3.7, 4.7, 5.7, 6.6, 7.6};
		Double_t itEndcapZpos[itNEndcap] = {15.5, 29.0, 43.5, 58.0, 72.5, 87.0, 101.5};

		for (Int_t i=0; i<itNBarrel; i++)
		{
			TGeoTube *itBarrelTube =
				new TGeoTube(Form("itBarrelTube%i",i), itBarrelInnerR[i], itBarrelInnerR[i] + itLayerThickness, itBarrelHalfZ[i]);
			TGeoVolume *itBarrelLayer = new TGeoVolume(Form("itBarrelLayer%i",i), itBarrelTube, pmIT);
			itBarrelLayer->SetLineColor(2);
			volExpHall->AddNode(itBarrelLayer, i);
		}
		for (Int_t i=0; i<itNEndcap; i++)
		{
			TGeoTube *itEndcapTube = new TGeoTube(Form("itEndcapTube%i",i), itEndcapRmin[i], itEndcapRmax[i], 0.5*itLayerThickness);
			TGeoVolume *itEndcapLayer = new TGeoVolume(Form("itEndcapLayer%i",i), itEndcapTube, pmIT);
			itEndcapLayer->SetLineColor(2);
			volExpHall->AddNode(itEndcapLayer, 2*i, new TGeoTranslation(0,0,itEndcapZpos[i]));
			volExpHall->AddNode(itEndcapLayer, 2*i+1, new TGeoTranslation(0,0,-itEndcapZpos[i]));
		}

	}

	if (viewTPC) {

		// TPC
		const Double_t tpcInnerSupportInnerR = 38.0;
		const Double_t tpcInnerSupportOuterR = 39.5;
		const Double_t tpcHalfZ = 260.0;
		const Double_t tpcOuterSupportOuterR = 206.0;
		const Double_t tpcOuterSupportHalfThickness = 4.1175;
		const Double_t tpcEndcapHalfThickness = 1.0;
		TGeoTube *tpcInnerSupportTube = new TGeoTube("tpcInnerSupportTube", tpcInnerSupportInnerR, tpcInnerSupportOuterR, tpcHalfZ);
		TGeoVolume *tpcInnerSupport = new TGeoVolume("tpcInnerSupport", tpcInnerSupportTube, pmTPCSupport);
		tpcInnerSupport->SetLineColor(4);
		volExpHall->AddNode(tpcInnerSupport,1);

		TGeoTube *tpcBarrelTube =
			new TGeoTube("tpcBarrelTube", tpcInnerSupportOuterR, tpcOuterSupportOuterR-2*tpcOuterSupportHalfThickness, tpcHalfZ-2*tpcEndcapHalfThickness);
		TGeoVolume *tpcBarrel = new TGeoVolume("tpcBarrel", tpcBarrelTube, pmTPCSupport);
		tpcBarrel->SetLineColor(4);
		volExpHall->AddNode(tpcBarrel, 1);

		TGeoTube *tpcEndcapTube =
			new TGeoTube("tpcEndcapTube", tpcInnerSupportOuterR, tpcOuterSupportOuterR-2*tpcOuterSupportHalfThickness, tpcEndcapHalfThickness);
		TGeoVolume *tpcEndcap = new TGeoVolume("tpcEndcap", tpcEndcapTube, pmTPCSupport);
		tpcEndcap->SetLineColor(4);
		volExpHall->AddNode(tpcEndcap, 1, new TGeoTranslation(0,0,tpcHalfZ-tpcEndcapHalfThickness));
		volExpHall->AddNode(tpcEndcap, 2, new TGeoTranslation(0,0,-tpcHalfZ+tpcEndcapHalfThickness));

		TGeoTube *tpcOuterSupportTube =
			new TGeoTube("tpcOuterSupportTube", tpcOuterSupportOuterR-2*tpcOuterSupportHalfThickness, tpcOuterSupportOuterR, tpcHalfZ);
		TGeoVolume *tpcOuterSupport = new TGeoVolume("tpcOutSupport", tpcOuterSupportTube, pmTPCSupport);
		tpcOuterSupport->SetLineColor(4);
		volExpHall->AddNode(tpcOuterSupport, 1);

	}

	if (viewCLX) {

		// EM calorimeter
		const Double_t ecalLayerThickness = 0.6;
		const Int_t ecalNLayers = 33;
		const Double_t ecalThickness = ecalNLayers * ecalLayerThickness;

		const Int_t ecalNTraps = 12;
		const Double_t ecalTrapDPhi = 360.0 / ecalNTraps; // in degrees
		const Double_t ecalTrapInnerR = 210.0;
		const Double_t ecalTrapOuterR = ecalTrapInnerR + ecalThickness;
		const Double_t ecalTrapMidR = ecalTrapInnerR + 0.5*ecalThickness;
		const Double_t ecalHalfZ = 270.0;

		TGeoTrd1 *ecalBlockTrd1 =
		new TGeoTrd1("ecalBlockTrd1",
			     ecalTrapInnerR*TMath::Tan(0.5*ecalTrapDPhi*TMath::DegToRad()),
			     ecalTrapOuterR*TMath::Tan(0.5*ecalTrapDPhi*TMath::DegToRad()),
			     ecalHalfZ,
			     0.5*ecalThickness);
		TGeoVolume *ecalBlock = new TGeoVolume("ecalBlock", ecalBlockTrd1, pmEMCAL);
		ecalBlock->SetLineColor(6);
		for (Int_t i=0; i<ecalNTraps; i++) {
			TGeoCombiTrans *ecalCombiTrans = new TGeoCombiTrans();
			ecalCombiTrans->RotateX(-90.0);
			ecalCombiTrans->RotateZ(-i*ecalTrapDPhi);
			ecalCombiTrans->SetTranslation(ecalTrapMidR*TMath::Sin(i*ecalTrapDPhi*TMath::DegToRad()),
						       ecalTrapMidR*TMath::Cos(i*ecalTrapDPhi*TMath::DegToRad()),
						       0.0);
			volExpHall->AddNode(ecalBlock, i+1, ecalCombiTrans);
		}

		// Endcap EM calorimeter
		const Double_t endEcalInnerR = 40.0;
		const Double_t endEcalOuterR = 220.0;
		const Double_t endEcalMidR = endEcalInnerR + 0.5*(endEcalOuterR - endEcalInnerR);
		const Double_t endEcalZpos = 280.0 + 0.5*ecalThickness;

		TGeoTrd1 *endEcalTrd1 =
		new TGeoTrd1("endEcalTrd1",
			     endEcalInnerR*TMath::Tan(0.5*ecalTrapDPhi*TMath::DegToRad()),
			     endEcalOuterR*TMath::Tan(0.5*ecalTrapDPhi*TMath::DegToRad()),
			     0.5*ecalThickness,
			     0.5*(endEcalOuterR-endEcalInnerR));
		TGeoVolume *endEcal = new TGeoVolume("endEcal", endEcalTrd1, pmEMCAL);
		endEcal->SetLineColor(6);
		for (Int_t i=0; i<ecalNTraps; i++) {
			TGeoTranslation tr1("endEcalTrans1",
					endEcalMidR*TMath::Sin(i*ecalTrapDPhi*TMath::DegToRad()),
					endEcalMidR*TMath::Cos(i*ecalTrapDPhi*TMath::DegToRad()),
					endEcalZpos);
			TGeoTranslation tr2("endEcalTrans2",
					endEcalMidR*TMath::Sin(i*ecalTrapDPhi*TMath::DegToRad()),
					endEcalMidR*TMath::Cos(i*ecalTrapDPhi*TMath::DegToRad()),
					-endEcalZpos);
			TGeoRotation rot("endEcalRot");
			rot.RotateX(-90.0);
			rot.RotateZ(-i*ecalTrapDPhi);
			volExpHall->AddNode(endEcal, 2*i, new TGeoCombiTrans(tr1, rot));
			volExpHall->AddNode(endEcal, 2*i+1, new TGeoCombiTrans(tr2, rot));
		}

		// Hadron calorimeter
		const Int_t hcalNLayers = 46;
		const Double_t hcalLayerThickness = 2.6;
		const Double_t hcalThickness = hcalNLayers * hcalLayerThickness;
		const Int_t hcalNTraps = ecalNTraps;
		const Double_t hcalTrapDPhi = ecalTrapDPhi;
		const Double_t hcalHalfZ = 289.8;
		const Double_t hcalTrapInnerR = ecalTrapOuterR;
		const Double_t hcalTrapOuterR = hcalTrapInnerR + hcalThickness;
		const Double_t hcalTrapMidR = hcalTrapInnerR + 0.5*hcalThickness;

		TGeoTrd1 *hcalBlockTrd1 =
		new TGeoTrd1("hcalBlockTrd1",
			hcalTrapInnerR*TMath::Tan(0.5*hcalTrapDPhi*TMath::DegToRad()),
			hcalTrapOuterR*TMath::Tan(0.5*hcalTrapDPhi*TMath::DegToRad()),
			hcalHalfZ,
			0.5*hcalThickness);
		TGeoVolume *hcalBlock = new TGeoVolume("hcalBlock", hcalBlockTrd1, pmHDCAL);
		hcalBlock->SetLineColor(7);
		for (Int_t i=0; i<hcalNTraps; i++) {
			TGeoCombiTrans *hcalCombiTrans = new TGeoCombiTrans();
			hcalCombiTrans->RotateX(-90.0);
			hcalCombiTrans->RotateZ(-i*hcalTrapDPhi);
			hcalCombiTrans->SetTranslation(hcalTrapMidR*TMath::Sin(i*hcalTrapDPhi*TMath::DegToRad()),
						       hcalTrapMidR*TMath::Cos(i*hcalTrapDPhi*TMath::DegToRad()),
						       0.0);
			volExpHall->AddNode(hcalBlock, i+1, hcalCombiTrans);
		}

		// Endcap hadron calorimeter
		const Double_t endHcalInnerR = 40.0;
		const Double_t endHcalOuterR = hcalTrapOuterR;
		const Double_t endHcalMidR = endHcalInnerR + 0.5*(endHcalOuterR - endHcalInnerR);
		const Double_t endHcalZpos = endEcalZpos + 0.5*ecalThickness + 0.5*hcalThickness;

		TGeoTrd1 *endHcalTrd1 =
		new TGeoTrd1("endHcalTrd1",
			endHcalInnerR*TMath::Tan(0.5*hcalTrapDPhi*TMath::DegToRad()),
			endHcalOuterR*TMath::Tan(0.5*hcalTrapDPhi*TMath::DegToRad()),
			0.5*hcalThickness,
			0.5*(endHcalOuterR-endHcalInnerR));
		TGeoVolume *endHcal = new TGeoVolume("endHcal", endHcalTrd1, pmHDCAL);
		endHcal->SetLineColor(7);
		for (Int_t i=0; i<hcalNTraps; i++) {
			TGeoTranslation tr1("endHcalTrans1",
				endHcalMidR*TMath::Sin(i*hcalTrapDPhi*TMath::DegToRad()),
				endHcalMidR*TMath::Cos(i*hcalTrapDPhi*TMath::DegToRad()),
				endHcalZpos);
			TGeoTranslation tr2("endHcalTrans2",
				endHcalMidR*TMath::Sin(i*hcalTrapDPhi*TMath::DegToRad()),
				endHcalMidR*TMath::Cos(i*hcalTrapDPhi*TMath::DegToRad()),
				-endHcalZpos);
			TGeoRotation rot("endHcalRot");
			rot.RotateX(-90.0);
			rot.RotateZ(-i*hcalTrapDPhi);
			volExpHall->AddNode(endHcal, 2*i, new TGeoCombiTrans(tr1, rot));
			volExpHall->AddNode(endHcal, 2*i+1, new TGeoCombiTrans(tr2, rot));
		}

	}

	if (viewSOL) {
		// Solenoid
		const Double_t solInnerR = 375.0;
		const Double_t solOuterR = 440.0;
		const Double_t solHalfZ = 475.0;
		const Double_t solCoilInnerR = 397.5;
		const Double_t solCoilOuterR = 402.5;
		const Double_t solCoilHalfZ = 430.0;

		TGeoTube *solTube = new TGeoTube("solTube", solInnerR, solOuterR, solHalfZ);
		TGeoVolume *solenoid = new TGeoVolume("solenoid", solTube, pmCoil);
		solenoid->SetLineColor(11);
		volExpHall->AddNode(solenoid, 1);

		TGeoTube *solCoilTube = new TGeoTube("solCoilTube", solCoilInnerR, solCoilOuterR, solCoilHalfZ);
		TGeoVolume *solCoil = new TGeoVolume("solCoil", solCoilTube, pmCoil);
		solCoil->SetLineColor(11);
		volExpHall->AddNode(solCoil, 1);
	}


	if (viewMUD) {
		// Muon barrel detector
		const Int_t muNTraps = 12;
		const Double_t muTrapDPhi = 360.0 / muNTraps;
		const Double_t muBarrelOuterR = 765.0;
		const Double_t muBarrelInnerR = 450.0;
		const Double_t muBarrelMidR = muBarrelInnerR + 0.5 * (muBarrelOuterR - muBarrelInnerR);
		const Double_t muBarrelHalfZ = 485.0;
		const Double_t muFrontEndInnerR = 40.0;
		const Double_t muFrontEndOuterR = 260.0;
		const Double_t muFrontEndFrontZ = 425.0;
		const Double_t muFrontEndThickness = muBarrelHalfZ - muFrontEndFrontZ;
		const Double_t muFrontEndMidR = muFrontEndInnerR + 0.5 * (muFrontEndOuterR - muFrontEndInnerR);
		const Double_t muEndcapFrontZ = muBarrelHalfZ;
		const Double_t muEndcapInnerR = muFrontEndInnerR;
		const Double_t muEndcapOuterR = muBarrelOuterR;
		const Double_t muEndcapThickness = 315.0;
		const Double_t muEndcapMidR = muEndcapInnerR + 0.5 * (muEndcapOuterR - muEndcapInnerR);

		TGeoTrd1 *muBarrelTrd1 =
			new TGeoTrd1("muBarrelTrd1",
				     muBarrelInnerR * TMath::Tan(0.5*muTrapDPhi*TMath::DegToRad()),
				     muBarrelOuterR * TMath::Tan(0.5*muTrapDPhi*TMath::DegToRad()),
				     muBarrelHalfZ,
				     0.5 * (muBarrelOuterR - muBarrelInnerR));
		TGeoVolume *muBarrel = new TGeoVolume("muBarrel", muBarrelTrd1, pmMUD);
		TGeoTrd1 *muFrontEndTrd1 =
			new TGeoTrd1("muFrontEndTrd1",
				     muFrontEndInnerR * TMath::Tan(0.5*muTrapDPhi*TMath::DegToRad()),
				     muFrontEndOuterR * TMath::Tan(0.5*muTrapDPhi*TMath::DegToRad()),
				     0.5 * muFrontEndThickness,
				     0.5 * (muFrontEndOuterR - muFrontEndInnerR));
		TGeoVolume *muFrontEnd = new TGeoVolume("muFrontEnd", muFrontEndTrd1, pmMUD);
		TGeoTrd1 *muEndcapTrd1 =
			new TGeoTrd1("muEndcapTrd1",
				     muEndcapInnerR * TMath::Tan(0.5*muTrapDPhi*TMath::DegToRad()),
				     muEndcapOuterR * TMath::Tan(0.5*muTrapDPhi*TMath::DegToRad()),
				     0.5 * muEndcapThickness,
				     0.5 * (muEndcapOuterR - muEndcapInnerR));
		TGeoVolume *muEndcap = new TGeoVolume("muEndcap", muEndcapTrd1, pmMUD);

		muBarrel->SetLineColor(13);
		muFrontEnd->SetLineColor(13);
		muEndcap->SetLineColor(13);

		for (Int_t i=0; i<muNTraps; i++) {
			// barrel MUD
			TGeoCombiTrans *muBarrelCT = new TGeoCombiTrans();
			muBarrelCT->RotateX(-90.0);
			muBarrelCT->RotateZ(-i*muTrapDPhi);
			muBarrelCT->SetTranslation(muBarrelMidR * TMath::Sin(i*muTrapDPhi*TMath::DegToRad()),
						     muBarrelMidR * TMath::Cos(i*muTrapDPhi*TMath::DegToRad()),
						     0.0);
			volExpHall->AddNode(muBarrel, i+1, muBarrelCT);
			// front endcap MUD
			TGeoRotation muFrontEndRot("muFrontEndRot");
			muFrontEndRot.RotateX(-90.0);
			muFrontEndRot.RotateZ(-i*muTrapDPhi);
			TGeoTranslation muFrontEndTr1("muFrontEndTr1",
						      muFrontEndMidR * TMath::Sin(i*muTrapDPhi*TMath::DegToRad()),
						      muFrontEndMidR * TMath::Cos(i*muTrapDPhi*TMath::DegToRad()),
						      muFrontEndFrontZ + 0.5*muFrontEndThickness);
			TGeoTranslation muFrontEndTr2("muFrontEndTr2",
						      muFrontEndMidR * TMath::Sin(i*muTrapDPhi*TMath::DegToRad()),
						      muFrontEndMidR * TMath::Cos(i*muTrapDPhi*TMath::DegToRad()),
						      -muFrontEndFrontZ - 0.5*muFrontEndThickness);
			volExpHall->AddNode(muFrontEnd, 2*i+1, new TGeoCombiTrans(muFrontEndTr1, muFrontEndRot));
			volExpHall->AddNode(muFrontEnd, 2*i+2, new TGeoCombiTrans(muFrontEndTr2, muFrontEndRot));
			// endcap MUD
			TGeoRotation muEndcapRot("muEndcapRot");
			muEndcapRot.RotateX(-90.0);
			muEndcapRot.RotateZ(-i*muTrapDPhi);
			TGeoTranslation muEndcapTr1("muEndcapTr1",
						    muEndcapMidR * TMath::Sin(i*muTrapDPhi*TMath::DegToRad()),
						    muEndcapMidR * TMath::Cos(i*muTrapDPhi*TMath::DegToRad()),
						    muEndcapFrontZ + 0.5*muEndcapThickness);
			TGeoTranslation muEndcapTr2("muEndcapTr2",
						    muEndcapMidR * TMath::Sin(i*muTrapDPhi*TMath::DegToRad()),
						    muEndcapMidR * TMath::Cos(i*muTrapDPhi*TMath::DegToRad()),
						    -muEndcapFrontZ - 0.5*muEndcapThickness);
			volExpHall->AddNode(muEndcap, 2*i+1, new TGeoCombiTrans(muEndcapTr1, muEndcapRot));
			volExpHall->AddNode(muEndcap, 2*i+2, new TGeoCombiTrans(muEndcapTr2, muEndcapRot));
		}
	}

	// Close geometry and draw
	geoMgr->CloseGeometry();
	geoMgr->SetNsegments(80);
	volExpHall->Draw();
#if 1
	TVirtualGeoPainter *painter = geoMgr->GetGeomPainter();
	painter->SetRaytracing(kTRUE);
	gPad->Modified();
	gPad->Update();
#endif
	return 0;
}
